@@ -1,31 +1,25 @@
 """Decorators for Flask route authentication and authorization.
 
-Configuration:
-    AXIOMS_AUDIENCE: Required. Expected audience claim in the JWT token.
-    AXIOMS_JWKS_URL: Optional. Full URL to JWKS endpoint (e.g.,
-        https://my-auth.domain.com/oauth2/.well-known/jwks.json).
-    AXIOMS_DOMAIN: Optional. Axioms domain name. Used for constructing the
-        JWKS URL if AXIOMS_JWKS_URL is not provided. Also used as fallback
-        for extracting namespaced roles/permissions claims.
+This module provides decorators for protecting Flask routes with JWT-based authentication
+and authorization. Supports scope-based, role-based, and permission-based access control
+with configurable claim names for different authorization servers.
 
-Note: Either AXIOMS_JWKS_URL or AXIOMS_DOMAIN must be configured.
-
-Claims handling:
-    - Scopes: Checked from standard 'scope' claim
-    - Roles: Checked from 'roles' claim, or 'https://{AXIOMS_DOMAIN}/claims/roles' if AXIOMS_DOMAIN is set
-    - Permissions: Checked from 'permissions' claim, or 'https://{AXIOMS_DOMAIN}/claims/permissions' if AXIOMS_DOMAIN is set
+For complete configuration documentation, see the Configuration section in the API reference.
 """
 
 from functools import wraps
-from flask import request
+
 from flask import current_app as app
+from flask import request
+
 from .error import AxiomsError
 from .token import (
+    check_permissions,
+    check_roles,
+    check_scopes,
+    get_claim_from_token,
     has_bearer_token,
     has_valid_token,
-    check_scopes,
-    check_roles,
-    check_permissions,
 )
 
 
@@ -33,6 +27,9 @@ def has_required_scopes(*required_scopes):
     """Decorator to enforce scope-based authorization.
 
     Checks if the authenticated user's token contains any of the required scopes.
+    Uses OR logic: the token must have at least ONE of the specified scopes.
+
+    To require ALL scopes (AND logic), chain multiple decorators.
 
     Args:
         *required_scopes: Variable length list of required scope strings.
@@ -43,12 +40,20 @@ def has_required_scopes(*required_scopes):
     Raises:
         AxiomsError: If token is missing or doesn't contain required scopes.
 
-    Example::
+    Example (OR logic - requires EITHER scope)::
 
         @app.route('/api/resource')
         @has_required_scopes('read:resource', 'write:resource')
         def protected_route():
             return {'data': 'protected'}
+
+    Example (AND logic - requires BOTH scopes via chaining)::
+
+        @app.route('/api/strict')
+        @has_required_scopes('read:resource')
+        @has_required_scopes('write:resource')
+        def strict_route():
+            return {'data': 'requires both scopes'}
     """
 
     def decorator(fn):
@@ -63,7 +68,11 @@ def has_required_scopes(*required_scopes):
                     },
                     401,
                 )
-            if check_scopes(payload.scope, required_scopes[0]):
+
+            # Get scope from configured claim names
+            token_scope = get_claim_from_token(payload, "SCOPE") or ""
+
+            if check_scopes(token_scope, required_scopes[0]):
                 return fn(*args, **kwargs)
             raise AxiomsError(
                 {
@@ -82,6 +91,9 @@ def has_required_roles(*view_roles):
     """Decorator to enforce role-based authorization.
 
     Checks if the authenticated user's token contains any of the required roles.
+    Uses OR logic: the token must have at least ONE of the specified roles.
+
+    To require ALL roles (AND logic), chain multiple decorators.
 
     Args:
         *view_roles: Variable length list of required role strings.
@@ -92,12 +104,20 @@ def has_required_roles(*view_roles):
     Raises:
         AxiomsError: If token is missing or doesn't contain required roles.
 
-    Example::
+    Example (OR logic - requires EITHER role)::
 
         @app.route('/admin/users')
         @has_required_roles('admin', 'superuser')
         def admin_route():
             return {'users': []}
+
+    Example (AND logic - requires BOTH roles via chaining)::
+
+        @app.route('/admin/critical')
+        @has_required_roles('admin')
+        @has_required_roles('superuser')
+        def critical_route():
+            return {'message': 'requires both roles'}
     """
 
     def decorator(fn):
@@ -113,16 +133,8 @@ def has_required_roles(*view_roles):
                     401,
                 )
 
-            # Check for roles in standard claim first, then namespaced claim
-            token_roles = getattr(payload, "roles", None)
-            if token_roles is None and app.config.get("AXIOMS_DOMAIN"):
-                token_roles = getattr(
-                    payload,
-                    "https://{}/claims/roles".format(app.config["AXIOMS_DOMAIN"]),
-                    [],
-                )
-            if token_roles is None:
-                token_roles = []
+            # Get roles from configured claim names
+            token_roles = get_claim_from_token(payload, "ROLES") or []
 
             if check_roles(token_roles, view_roles[0]):
                 return fn(*args, **kwargs)
@@ -143,6 +155,9 @@ def has_required_permissions(*view_permissions):
     """Decorator to enforce permission-based authorization.
 
     Checks if the authenticated user's token contains any of the required permissions.
+    Uses OR logic: the token must have at least ONE of the specified permissions.
+
+    To require ALL permissions (AND logic), chain multiple decorators.
 
     Args:
         *view_permissions: Variable length list of required permission strings.
@@ -153,12 +168,20 @@ def has_required_permissions(*view_permissions):
     Raises:
         AxiomsError: If token is missing or doesn't contain required permissions.
 
-    Example::
+    Example (OR logic - requires EITHER permission)::
 
-        @app.route('/api/delete')
-        @has_required_permissions('resource:delete')
-        def delete_route():
-            return {'status': 'deleted'}
+        @app.route('/api/resource')
+        @has_required_permissions('resource:read', 'resource:write')
+        def resource_route():
+            return {'data': 'success'}
+
+    Example (AND logic - requires BOTH permissions via chaining)::
+
+        @app.route('/api/critical')
+        @has_required_permissions('resource:read')
+        @has_required_permissions('resource:admin')
+        def critical_route():
+            return {'message': 'requires both permissions'}
     """
 
     def decorator(fn):
@@ -174,16 +197,8 @@ def has_required_permissions(*view_permissions):
                     401,
                 )
 
-            # Check for permissions in standard claim first, then namespaced claim
-            token_permissions = getattr(payload, "permissions", None)
-            if token_permissions is None and app.config.get("AXIOMS_DOMAIN"):
-                token_permissions = getattr(
-                    payload,
-                    "https://{}/claims/permissions".format(app.config["AXIOMS_DOMAIN"]),
-                    [],
-                )
-            if token_permissions is None:
-                token_permissions = []
+            # Get permissions from configured claim names
+            token_permissions = get_claim_from_token(payload, "PERMISSIONS") or []
 
             if check_permissions(token_permissions, view_permissions[0]):
                 return fn(*args, **kwargs)
